@@ -1,13 +1,20 @@
 import functools
-from abc import ABC
-from distutils.command.build import build
 from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, TypeVar
 
 import instancelib as il
-from instancelib.machinelearning.sklearn import SkLearnClassifier
 from instancelib.typehints.typevars import DT, KT, LMT, LT, PMT, RT, VT
-from sklearn.base import ClassifierMixin, TransformerMixin
-from sklearn.linear_model import LogisticRegression
+
+from allib.estimation.catalog import EstimatorCatalog
+from allib.stopcriterion.catalog import StopCriterionCatalog
+
+from ..estimation.autostop import (
+    HorvitzThompsonLoose,
+    HorvitzThompsonVar1,
+    HorvitzThompsonVar2,
+)
+from ..stopcriterion.estimation import Conservative, Optimistic
+
+from ..stopcriterion.heuristic import LabelCount
 
 from ..environment.base import AbstractEnvironment
 from ..factory import AbstractBuilder, ObjectFactory
@@ -15,22 +22,39 @@ from ..machinelearning import AbstractClassifier, MachineLearningFactory
 from ..module.component import Component
 from ..typehints.typevars import IT
 from .autostop import AutoStopLearner
-from .autotar import AutoTarLearner, BinaryTarLearner
+from .autotar import AutoTarLearner, BinaryTarLearner, IncreasingBatch
 from .base import ActiveLearner
 from .catalog import ALCatalog as AL
 from .ensembles import StrategyEnsemble
 from .estimator import CycleEstimator, Estimator, RetryEstimator
 from .labelmethods import LabelProbabilityBased
 from .ml_based import ProbabilityBased
-from .mostcertain import (LabelMaximizer, LabelMaximizerNew,
-                          MostCertainSampling, MostConfidence)
-from .prob_ensembles import (LabelMinProbEnsemble, LabelProbEnsemble,
-                             ProbabilityBasedEnsemble)
+from .mostcertain import (
+    LabelMaximizer,
+    LabelMaximizerNew,
+    MostCertainSampling,
+    MostConfidence,
+)
+from .prob_ensembles import (
+    LabelMinProbEnsemble,
+    LabelProbEnsemble,
+    ProbabilityBasedEnsemble,
+)
 from .random import RandomSampling
 from .selectioncriterion import AbstractSelectionCriterion
-from .uncertainty import (EntropySampling, LabelUncertainty,
-                          LabelUncertaintyNew, LeastConfidence, MarginSampling,
-                          NearDecisionBoundary, RandomMLStrategy)
+from .uncertainty import (
+    EntropySampling,
+    LabelUncertainty,
+    LabelUncertaintyNew,
+    LeastConfidence,
+    MarginSampling,
+    NearDecisionBoundary,
+    RandomMLStrategy,
+)
+
+from .autotarensemble import AutoTARFirstMethod
+from .target import TargetMethod
+from .autostoplarge import AutoStopLarge
 
 
 class FallbackBuilder(AbstractBuilder):
@@ -87,7 +111,7 @@ class LabelProbabilityBasedBuilder(AbstractBuilder):
         return LabelProbabilityBased.builder(
             classifier,
             selection_criterion,
-            built_fallback,            
+            built_fallback,
             identifier=identifier,
         )
 
@@ -98,6 +122,7 @@ class PoolbasedBuilder(AbstractBuilder):
     ):
         return self._factory.create(query_type, identifier=identifier, **kwargs)
 
+
 class CustomBuilder(AbstractBuilder):
     def __call__(self, method: AL.CustomMethods, **kwargs):
         return self._factory.create(method, **kwargs)
@@ -245,35 +270,119 @@ def classifier_builder(
 
     return wrap_func
 
+
 class BinaryTarBuilder(AbstractBuilder):
-    def __call__(self, 
-                 machinelearning: Mapping[str, Any],
-                 batch_size: int,
-                 chunk_size: int = 2000, **kwargs):
+    def __call__(
+        self,
+        machinelearning: Mapping[str, Any],
+        batch_size: int,
+        chunk_size: int = 2000,
+        **kwargs,
+    ):
         classifier = self._factory.create(Component.CLASSIFIER, **machinelearning)
         return BinaryTarLearner.builder(classifier, batch_size, chunk_size, **kwargs)
 
 
+class IncreasingBatchBuilder(AbstractBuilder):
+    def __call__(
+        self,
+        machinelearning: Mapping[str, Any],
+        batch_size: int,
+        chunk_size: int = 2000,
+        **kwargs,
+    ):
+        classifier = self._factory.create(Component.CLASSIFIER, **machinelearning)
+        return IncreasingBatch.builder(classifier, batch_size, chunk_size, **kwargs)
+
+
 class AutoTARBuilder(AbstractBuilder):
-    def __call__(self, 
-                 machinelearning: Mapping[str, Any],
-                 k_sample: int, 
-                 batch_size: int, **kwargs):
+    def __call__(
+        self,
+        machinelearning: Mapping[str, Any],
+        k_sample: int,
+        batch_size: int,
+        **kwargs,
+    ):
         builder = self._factory.create(Component.CLASSIFIER, **machinelearning)
         at = AutoTarLearner.builder(builder, k_sample, batch_size, **kwargs)
         return at
 
+
+class PriorAutoTARBuilder(AbstractBuilder):
+    def __call__(
+        self,
+        tarmethod: Mapping[str, Any],
+        ensemble: Mapping[str, Any],
+        nneg: int = 50,
+        nirel: int = 10,
+        **kwargs,
+    ):
+        def stopbuilder(pos_label: LT, neg_label: LT):
+            return LabelCount(pos_label, 10)
+
+        tarbuilder = self._factory.create(Component.ACTIVELEARNER, **tarmethod)
+        ensbuilder = self._factory.create(Component.ACTIVELEARNER, **ensemble)
+        builder = AutoTARFirstMethod.builder(tarbuilder, ensbuilder, stopbuilder, nirel)
+        return builder
+
+
+class TargetBuilder(AbstractBuilder):
+    def __call__(
+        self,
+        tarmethod: Mapping[str, Any],
+        nrel: int = 10,
+        **kwargs,
+    ):
+        tarbuilder = self._factory.create(Component.ACTIVELEARNER, **tarmethod)
+        builder = TargetMethod.builder(tarbuilder, nrel)  # type: ignore
+        return builder
+
+
 class AutoSTOPBuilder(AbstractBuilder):
-    def __call__(self, 
-                 machinelearning: Mapping[str, Any],
-                 k_sample: int, 
-                 batch_size: int, 
-                 **kwargs):
+    def __call__(
+        self,
+        machinelearning: Mapping[str, Any],
+        k_sample: int,
+        batch_size: int,
+        **kwargs,
+    ):
         builder = self._factory.create(Component.CLASSIFIER, **machinelearning)
         at = AutoStopLearner.builder(builder, k_sample, batch_size, **kwargs)
         return at
 
 
+ESTIMATORS = {
+    EstimatorCatalog.HorvitzThompsonLoose: HorvitzThompsonLoose,
+    EstimatorCatalog.HorvitzThompson1: HorvitzThompsonVar1,
+    EstimatorCatalog.HorvitzThompson2: HorvitzThompsonVar2,
+}
+CRITERIA = {
+    StopCriterionCatalog.CONSERVATIVE: Conservative.builder,
+    StopCriterionCatalog.OPTIMISTIC: Optimistic.builder,
+}
+
+
+class AutoSTOPLargeBuilder(AbstractBuilder):
+    def __call__(
+        self,
+        machinelearning: Mapping[str, Any],
+        k_sample: int,
+        batch_size: int,
+        estimator: EstimatorCatalog,
+        stopcriterion: StopCriterionCatalog,
+        target: float,
+        **kwargs,
+    ):
+        builder = self._factory.create(Component.CLASSIFIER, **machinelearning)
+        estimator_builder = ESTIMATORS[estimator]
+        criterion_builder = CRITERIA[stopcriterion]
+
+        at = AutoStopLarge.builder(
+            builder, k_sample, batch_size, estimator_builder, criterion_builder, target
+        )
+        return at
+
+
 class ActiveLearningFactory(ObjectFactory):
     def __init__(self) -> None:
         super().__init__()
@@ -303,8 +412,13 @@ class ActiveLearningFactory(ObjectFactory):
         self.register_builder(
             AL.Paradigm.LABEL_MIN_PROB_ENSEMBLE, LabelMinProbilityBasedEnsembleBuilder()
         )
+        self.register_builder(AL.CustomMethods.PRIORAUTOTAR, PriorAutoTARBuilder())
+        self.register_builder(AL.CustomMethods.TARGET, TargetBuilder())
         self.register_builder(AL.CustomMethods.AUTOTAR, AutoTARBuilder())
         self.register_builder(AL.CustomMethods.BINARYTAR, BinaryTarBuilder())
+        self.register_builder(
+            AL.CustomMethods.INCREASING_BATCH, IncreasingBatchBuilder()
+        )
         self.register_builder(AL.CustomMethods.AUTOSTOP, AutoSTOPBuilder())
         self.register_constructor(AL.QueryType.RANDOM_SAMPLING, RandomSampling.builder)
         self.register_constructor(AL.QueryType.LEAST_CONFIDENCE, LeastConfidence)
@@ -322,4 +436,3 @@ class ActiveLearningFactory(ObjectFactory):
             AL.QueryType.LABELUNCERTAINTY_NEW, LabelUncertaintyNew
         )
         self.register_constructor(AL.QueryType.RANDOM_ML, RandomMLStrategy)
-  
